"""
These are a set of utility services that can be used for things like capturing
segmented video clips, FPS speeds, interfacing with your webcam, saving
event-based triggered videos, and image manipulations.
"""

from enum import Enum

import cv2





################################################################################
#                          Video Services                                      #
################################################################################

class CameraFailedToStart(RuntimeError):
    """
    Error when camera initialization fails.
    """
    def __init__(self, cmd, backend, append=''):
        pass

    def __str__(self):
        pass

class CameraConnectionLost(RuntimeError):
    """
    Error for when the camera connection is not found after the connection has been established.
    """
    def __init__(self):
        pass

class WebcamVideoStream(_BaseVideoStream):
    """
    Capture video frames from a webcam or CSI camera attached to your device.

    `WebcamVideoStream` can be instantiated as a context manager::

        with edgeiq.WebcamVideoStream() as video_stream:
            ...

    To use `WebcamVideoStream` directly, use the
    :func:`~edgeiq.edge_tools.WebcamVideoStream.start()` and
    :func:`~edgeiq.edge_tools.WebcamVideoStream.stop()` functions::

        video_stream = edgeiq.WebcamVideoStream().start()
        ...
        video_stream.stop()

    Typical usage::

        with edgeiq.WebcamVideoStream() as video_stream:
            while True:
                frame = video_stream.read()

    :type cam: integer
    :param cam: The integer identifier of the camera.
    """
    def __init__(self, cam=0):
        pass

class GStreamerVideoStream(_BaseVideoStream):
    """
    Capture video frames using the gstreamer plugin.

    This can be useful for capturing videos from CSI cameras when
    V4L is not supported.

    `GStreamerVideoStream` can be instantiated as a context manager::

        with edgeiq.GStreamerVideoStream() as video_stream:
            ...

    To use `GStreamerVideoStream` directly, use the
    :func:`~edgeiq.edge_tools.GStreamerVideoStream.start()` and
    :func:`~edgeiq.edge_tools.GStreamerVideoStream.stop()` functions::

        video_stream = edgeiq.GStreamerVideoStream().start()
        ...
        video_stream.stop()

    Typical usage::

        with edgeiq.GStreamerVideoStream() as video_stream:
            while True:
                frame = video_stream.read()

    :type cam: integer
    :param cam: The integer identifier of the camera.
    :type display_width: integer
    :param display_width: The output image width in pixels.
    :type display_height: integer
    :param display_height: The output image height in pixels.
    :type framerate: integer
    :param framerate: The recording frame rate.
    """
    def __init__(
            self, cam=0, display_width=640, display_height=480, framerate=30):
        pass

class FrameRotation(Enum):
    """Amount of rotation applied to each frame in degrees"""
    ROTATE_NONE = 0
    ROTATE_90 = 90
    ROTATE_180 = 180


class JetsonCameraMode(Enum):
    """
    Sensor Mode applied to CSI camera which determines input width and height
    and framerate.  The first Number identifies the Sony Sensor Number (IMX219
    or IMX477).  The second numbers are the input width and height.  The third
    number is the framerate and fourth the number is the camera sensor mode.
    """
    IMX219_3264x2468_21_0 = 0
    IMX219_3264x1848_28_1 = 1
    IMX219_1920x1080_30_2 = 2
    IMX219_1640x1232_30_3 = 3
    IMX477_4032x3040_30_0 = 4
    IMX477_1920x1080_60_1 = 5
    IMX477_2560x1440_40_3 = 7


class JetsonVideoStream(_BaseVideoStream):
    """
    Capture video frames from a CSI ribbon camera on NVIDIA Jetson.

    `JetsonVideoStream` can be instantiated as a context manager::

        with edgeiq.JetsonVideoStream() as video_stream:
            ...

    To use `JetsonVideoStream` directly, use the
    :func:`~edgeiq.edge_tools.JetsonVideoStream.start()` and
    :func:`~edgeiq.edge_tools.JetsonVideoStream.stop()` functions::

        video_stream = edgeiq.JetsonVideoStream().start()
        ...
        video_stream.stop()

    Typical usage::

        with edgeiq.JetsonVideoStream() as video_stream:
            while True:
                frame = video_stream.read()

    :type cam: integer
    :param cam: The integer identifier of the camera.
    :type rotation: :class:`~FrameRotation`
    :param rotation: The rotation applied to each frame
    :type camera_mode: :class:`~JetsonCameraMode`
    :param camera_mode: The sensor mode for csi camera
    :type display_width: integer
    :param display_width: The output image width in pixels.
    :type display_height: integer
    :param display_height: The output image height in pixels.
    """
    def __init__(
            self, cam=0, rotation=FrameRotation.ROTATE_NONE,
            camera_mode=JetsonCameraMode.IMX219_1920x1080_30_2,
            display_width=640, display_height=480):
        pass

class IPVideoStream(_BaseVideoStream):
    """
    Capture video frames from an IP stream.

    `IPVideoStream` can be instantiated as a context manager::

        with edgeiq.IPVideoStream('tcp://0.0.0.0:3333') as video_stream:
            ...

    To use `IPVideoStream` directly, use the
    :func:`~edgeiq.edge_tools.IPVideoStream.start()` and
    :func:`~edgeiq.edge_tools.IPVideoStream.stop()` functions::

        video_stream = edgeiq.IPVideoStream('tcp://0.0.0.0:3333').start()
        ...
        video_stream.stop()

    Typical usage::

        with edgeiq.IPVideoStream('tcp://0.0.0.0:3333') as video_stream:
            while True:
                frame = video_stream.read()

    :type url: string
    :param url: The URL of the IP stream
    """
    def __init__(self, url):
        pass

class FPS:
    """
    Monitor the frames per second (FPS) of video streams for performance tracking.

    Typical usage::

        fps = edgeiq.FPS().start()
        while True:
            <main processing loop>
            fps.update()

        # Get the elapsed time and FPS
        fps.stop()
        print("Elapsed seconds: {}".format(fps.get_elapsed_seconds()))
        print("FPS: {}".format(fps.compute_fps()))

    :func:`~edgeiq.edge_tools.FPS.compute_fps()` may also be called in the main
    processing loop to compute an instantaneous estimate of the FPS.
    """
    def __init__(self):
        pass

    def start(self):
        pass

    def stop(self):
        pass

    def update(self):
        pass

    def get_elapsed_seconds(self):
        pass

    def compute_fps(self):
        pass

class NoMoreFrames(RuntimeError):
    """
    Error when trying to read more frames but no frames remaining
    to read.
    """
    def __init__(self):
        pass

class FileVideoStream:
    """
    Stream a video file for analysis.

    `FileVideoStream` can be instantiated as a context manager::

        with edgeiq.FileVideoStream('/path/to/video.mp4') as video_stream:
            ...

    To use `FileVideoStream` directly, use the
    :func:`~edgeiq.edge_tools.FileVideoStream.start()` and
    :func:`~edgeiq.edge_tools.FileVideoStream.stop()` functions::

        video_stream = edgeiq.FileVideoStream('/path/to/video.mp4').start()
        ...
        video_stream.stop()

    Typical usage::

        with edgeiq.FileVideoStream('/path/to/video.mp4') as video_stream:
            while video_stream.more():
                frame = video_stream.read()

    :type path: string
    :param path: The path of the video file location.
    :type queue_size: integer
    :param queue_size: The size of the buffer queue.
    :type play_realtime: boolean
    :param play_realtime: If `True` play video in simulated realtime,
                          where frames are dropped as they become outdated.
    """
    def __init__(self, path, queue_size=128, play_realtime=False):
        pass

    def start(self):
        pass

    def read(self):
        pass

    def more(self):
        pass

    def stop(self):
        pass

    def __enter__(self):
        pass

    def __exit__(self, type, value, traceback):
        pass

    def __del__(self):
        pass

class VideoWriter:
    """
    Save a video clip to a file.

    `VideoWriter` can be instantiated as a context manager::

        with edgeiq.VideoWriter('/path/to/output/video.avi') as video_writer:
            ...

    To use `VideoWriter` directly, make sure to call
    :func:`~edgeiq.edge_tools.VideoWriter.close()` when usage is complete::

        video_writer = edgeiq.VideoWriter('/path/to/output/video.avi')
        ...
        video_writer.close()

    Typical usage::

        with edgeiq.VideoWriter('/path/to/output/video.avi') as video_writer:
            while True:
                <read and process frame>
                video_writer.write_frame(frame)

    :type output_path: string
    :param output_path: The path to save the video file at.
    :type fps: integer
    :param fps: The number of frames per second to save.
    :type color: boolean
    :param color: If `True`, save the video in color.
    :type codec: string
    :param codec: The video codec to use for saving the video clip.
    """
    def __init__(
            self, output_path="output.avi", fps=30, color=True,
            codec='MJPG'):
        pass

    def __del__(self):
        pass

    def write_frame(self, frame):
        pass

    def close(self):
        pass

    def __enter__(self):
        pass

    def __exit__(self, type, value, traceback):
        pass

class EventVideoWriterState(Enum):
    """States of the `EventVideoWriter`"""
    IDLE = 'Idle'
    RECORDING = 'Recording'
    POST_ROLL = 'Post-roll'


class EventVideoWriter:
    """
    Save a segment of a video stream based upon an event trigger.

    `EventVideoWriter` can be instantiated as a context manager::

        with edgeiq.EventVideoWriter() as video_writer:
            ...

    To use `EventVideoWriter` directly, make sure to call
    :func:`~edgeiq.edge_tools.EventVideoWriter.close()` when
    usage is complete::

        video_writer = edgeiq.EventVideoWriter()
        ...
        video_writer.close()

    Typical usage::

        def video_writer_callback(event_name):
            print('Event {} has completed'.format(event_name))


        with edgeiq.EventVideoWriter() as video_writer:
            while True:
                <read and process frame>
                ...
                video_writer.update(frame)

                if event_started:
                    video_writer.start_event(
                        '/path/to/output/video.avi', video_writer_callback, 'Event 1')

                if event_ended:
                    video_writer.finish_event()

    :type pre_roll: integer
    :param pre_roll: The number of frames to capture preceding the event
                     trigger.
    :type post_roll: integer
    :param post_roll: The number of frames to capture after the event
                      completes.
    :type fps: integer
    :param fps: The number of frames per second to save.
    :type color: boolean
    :param color: If `True`, save the video with color.
    :type codec: string
    :param codec: The video codec to use for saving the video clip.
    """
    def __init__(self, pre_roll=150, post_roll=150, fps=30, color=True, codec='MJPG'):
        pass

    @property
    def state(self):
        pass

    def update(self, frame):
        pass

    def start_event(
            self, output_path, callback_function=None, callback_args=()):
        pass

    def finish_event(self):
        pass

    def close(self):
        pass

    def __enter__(self):
        pass

    def __exit__(self, type, value, traceback):
        pass

def translate(image, x, y):
    pass

def rotate(image, angle, center=None, scale=1.0):
    pass

def resize(image, width=None, height=None, keep_scale=True, inter=cv2.INTER_AREA):
    pass

def list_images(base_path, contains=None):
    pass

def list_files(
        base_path,
        valid_exts=(".jpg", ".jpeg", ".png", ".bmp", ".tif", ".tiff"),
        contains=None):
    pass

def markup_image(
        image, predictions, show_labels=True, show_confidences=True,
        colors=None, line_thickness=2, font_size=0.5, font_thickness=2):
    pass

def transparent_overlay_boxes(
        image, predictions, alpha=0.5, colors=None,
        show_labels=False, show_confidences=False):
    pass

def filter_predictions_by_label(predictions, label_list):
    pass

def filter_predictions_by_area(predictions, min_area_thresh):
    pass

def pad_to_aspect_ratio(image, a_ratio):
    pass

def find_usb_device(id_vendor, id_product):
    pass

def find_ncs2():
    pass

def is_jetson():
    pass

def is_jetson_nano():
    pass

def is_jetson_tx2():
    pass

def is_jetson_xavier_nx():
    pass

def cutout_image(image, box):
    pass

def blur_objects(image, predictions):
    pass

def blend_images(foreground_image, background_image, alpha):
    pass

def overlay_image(foreground_image, background_image, foreground_mask):
    pass

