from enum import Enum

import depthai



CAMERA_NODE_NAME = "color stream"
NETWORK_NODE_NAME = "neural network stream"
DEPTH_NODE_NAME = "depth stream"
CONTROLLER_NAME = "controller"


class Sensor(Enum):
    '''
    Sensors supported by the Oak Camera.
    '''
    res_1080 = depthai.ColorCameraProperties.SensorResolution.THE_1080_P
    res_4k = depthai.ColorCameraProperties.SensorResolution.THE_4_K
    res_12_mp = depthai.ColorCameraProperties.SensorResolution.THE_12_MP


class VideoMode(Enum):
    '''
    Video modes supported by the Oak Camera.

    VideoMode.preview - view from the preview feed which is the image that is fed into the neural network.

    VideoMode.video - view from the actual sensor to get a full resolution image.
    '''
    preview = 1
    video = 2


SENSOR_MAX_RESOLUTION = {
    Sensor.res_1080: (1920, 1080),
    Sensor.res_4k: (3840, 2160),
    Sensor.res_12_mp: (3840, 2160)
}

AVAILABLE_SENSORS = [Sensor.res_1080, Sensor.res_4k, Sensor.res_12_mp]


class Oak:
    """
    Class to be used to with Oak cameras to capture frames and execute neural network models.

    Typical usage::

        with edgeiq.Oak('alwaysai/mobilenet_ssd_oak') as camera:

            while True:
                frame = camera.get_frame()
                result = camera.get_model_result(confidence_level=.75)
                if result:
                    frame = edgeiq.markup_image(frame, result.predictions)

    :type model_id: string
    :param model_id: The ID of the model you want to initialize on the camera.
                     Supported model purposes: Object Detection, Classification, Pose Estimation.
                     Intializing the camera with no model_id or supplying None will still allow you to capture frames.

    :type sensor: :class:`.Sensor`
    :param sensor: The sensor to use on the camera.

    :type video_mode: :class:`.VideoMode`
    :param video_mode: The video output mode for :func:`~Oak.get_frame`.

    :type capture_depth: bool
    :param capture_depth: Set to capture the depth stream on OAK-D cameras. Depth stream can be retrieved by calling :func:`Oak.get_depth`.

    :type usb2: bool
    :param usb2: Set to use the device on USB2.

    :type device_info: depthai.DeviceInfo
    :param device_info: The device info for the target camera.
    """
    __DEVICES_IN_USE = []

    def __init__(self,
                 model_id=None,
                 sensor=Sensor.res_1080,
                 video_mode=VideoMode.video,
                 capture_depth=False,
                 usb2=False,
                 device_info=None):
        pass

    @property
    def model_id(self):
        pass

    @property
    def model_purpose(self):
        pass

    @classmethod
    def get_devices_in_use(cls):
        pass

    @staticmethod
    def get_available_devices():
        pass

    def get_frame(self):
        pass

    def get_model_result(self, block=True, **inference_args):
        pass

    def get_model_data(self, block=True):
        pass

    def get_depth(self, block=True):
        pass

    def start(self):
        pass

    def stop(self):
        pass

    def __enter__(self):
        pass

    def __exit__(self, type, value, traceback):
        pass

